import java.util.ArrayList;
import java.io.IOException; 

/******************************************************************************
*  Index of available Spells.
*
*  @author   Daniel R. Collins (dcollins@superdan.net)
*  @since    2017-06-17
*  @version  1.0
******************************************************************************/

public class SpellIndex {

	//--------------------------------------------------------------------------
	//  Constants
	//--------------------------------------------------------------------------

	/** Name of file with record data. */
	static final String DATA_FILE = "SpellIndex.csv";

	//--------------------------------------------------------------------------
	//  Fields
	//--------------------------------------------------------------------------

	/** The singleton class instance. */
	static SpellIndex instance = null;
	
	/** Array of individual records. */
	Spell[] array;

	/** Maximum spell level. */
	int maxLevel;

	/** Count of spells at each level. */
	int[] spellsPerLevel;

	/** Count of spells with positive weights per level. */
	int[] positiveWeights;

	/** Sum of frequency weights at each level. */
	int[] sumWeights;

	//--------------------------------------------------------------------------
	//  Constructors
	//--------------------------------------------------------------------------

	/**
	*  Constructor (read from dedicated file).
	*/
	protected SpellIndex () throws IOException {
		String[][] table = CSVReader.readFile(DATA_FILE);
		array = new Spell[table.length - 1];
		for (int i = 0; i < table.length - 1; i++)
			array[i] = new Spell(table[i + 1]);
		findMaxLevel();		
		countSpellsPerLevel();
		countPositiveWeights();
		sumWeights();
	}

	//--------------------------------------------------------------------------
	//  Methods
	//--------------------------------------------------------------------------

	/**
	*  Access the singleton class instance.
	*/
	public static SpellIndex getInstance() {
		if (instance == null) {
			try {
				instance = new SpellIndex();
			}
			catch (IOException e) {
				System.err.println("Failed to read file " + DATA_FILE);
			}
		}
		return instance;
	}

	/**
	*  Search for the maximum spell level.
	*/
	private void findMaxLevel () {
		maxLevel = -1;
		for (Spell s: array)
			if (s.getLevel() > maxLevel)
				maxLevel = s.getLevel();
	}

	/**
	*  Count spells at each level.
	*/
	private void countSpellsPerLevel () {
		spellsPerLevel = new int[maxLevel];
		for (Spell s: array)
			spellsPerLevel[s.getLevel() - 1]++;
	}

	/**
	*  Count spells with positive weights at each level.
	*/
	private void countPositiveWeights () {
		positiveWeights = new int[maxLevel];
		FreqWeightTable fwt = FreqWeightTable.getInstance();
		for (Spell s: array)
			if (fwt.getWeight(s.getFrequency()) > 0)
				positiveWeights[s.getLevel() - 1]++;
	}

	/**
	*  Sum frequency weights at each level.
	*/
	private void sumWeights () {
		sumWeights = new int[maxLevel];
		FreqWeightTable fwt = FreqWeightTable.getInstance();
		for (Spell s: array)
			sumWeights[s.getLevel() - 1] 
				+= fwt.getWeight(s.getFrequency());
	}

	/**
	*  Get maximum spell level.
	*/
	public int getMaxLevel() { 
		return maxLevel;
	}

	/**
	*  Get number of spells at a given level.
	*  @param level The spell level (1-based).
	*/
	public int getSpellsPerLevel (int level) { 
		return spellsPerLevel[level - 1];
	}

	/**
	*  Get number of spells with positive weighting at a given level.
	*  @param level The spell level (1-based).
	*/
	public int getPositiveWeights (int level) {
		return positiveWeights[level - 1];
	}

	/**
	*  Get sum frequency weight for a given level.
	*  @param level The spell level (1-based).
	*/
	public int getSumWeight (int level) {
		return sumWeights[level - 1];	
	}

	/**
	*  Get a spell by level and index.
	*  @param level The spell level (1-based).
	*  @param index Index of the spell (1-based).
	*/
	public Spell getSpell (int level, int index) {
		int indexCount = index;
		for (Spell s: array) {
			if (s.getLevel() == level)
				if (--indexCount == 0)
					return s;
		}
		System.err.println("Error: Could not match spell"
			+ " level " + level + ", index " + index);
		return null;
	}

	/**
	*  Get a spell by level and weighted index.
	*  @param level The spell level (1-based).
	*  @param weightIdx Weighted index of the spell (1-based).
	*/
	public Spell getSpellByWeight (int level, int weightIdx) {
		int weightIdxCount = weightIdx;
		FreqWeightTable fwt = FreqWeightTable.getInstance();
		for (Spell s: array) {
			if (s.getLevel() == level) {
				int weight = fwt.getWeight(s.getFrequency());
				if (weight > 0) {
					weightIdxCount -= weight;
					if (weightIdxCount <= 0)
						return s;				
				}
			}
		}
		System.err.println("Error: Could not match spell"
			+ " level " + level + ", weightIdx " + weightIdx);
		return null;
	}

	/**
	*  Main test method.
	*/
	public static void main (String[] args) {
		SpellIndex index = SpellIndex.getInstance();
		for (int level = 1; level <= index.getMaxLevel(); level++) {
			System.out.println("\nLevel " + level + "\n-------");
			for (int i = 1; i <= index.getSpellsPerLevel(level); i++)
				System.out.println(index.getSpell(level, i));
		}
	}
}

