import java.util.ArrayList;
import java.util.Collections;

/******************************************************************************
*  One OD&D wizard spellbook.
*
*  @author   Daniel R. Collins (dcollins@superdan.net)
*  @since    2017-06-18
*  @version  1.02
******************************************************************************/

public class Spellbook {

	//--------------------------------------------------------------------------
	//  Fields
	//--------------------------------------------------------------------------

	/** Wizard level */
	int wizLevel;
	
	/** Wizard intelligence */
	int wizIntel;

	/** Spells by level */
	ArrayList<ArrayList<Spell>> spellList;

	//--------------------------------------------------------------------------
	//  Constructors
	//--------------------------------------------------------------------------

	/**	
	*  Constructor (level, intel).
	*/
	Spellbook (int level, int intel) {
		wizLevel = level;
		wizIntel = intel;
		addAllSpellLevels();
	}

	//--------------------------------------------------------------------------
	//  Methods
	//--------------------------------------------------------------------------

	/**
	*  Add all spell levels to the book.
	*  Spell increment per level is effectively slope, m = dy/dx.
	*/
	void addAllSpellLevels() {
		spellList = new ArrayList<ArrayList<Spell>>();
		int maxLevel = computeMaxLevel();
		int minSpells = IntTable.getInstance().getMinSpells(wizIntel);
		int maxSpells = IntTable.getInstance().getMaxSpells(wizIntel);
		float spellsIncLevel = (float)(maxSpells - minSpells) / (maxLevel - 1);
		float spellsNextLevel = maxSpells;
		for (int level = 1; level <= maxLevel; level++) {
			addOneSpellLevel(level, (int) spellsNextLevel);
			spellsNextLevel -= spellsIncLevel;
		}
	}

	/**
	*  Add one spell level to the book.
	*/
	void addOneSpellLevel (int level, int num) {

		// Collect required data
		ArrayList<Spell> newLevel = new ArrayList<Spell>();
		SpellIndex index = SpellIndex.getInstance();
		num = Math.min(num, index.getPositiveWeights(level));
		int sumWeight = index.getSumWeight(level);

		// Loop over each spell
		for (int i = 0; i < num; i++) {
			Spell spell;
			do {
				int roll = Dice.roll(sumWeight);
				spell = index.getSpellByWeight(level, roll);
			} while (newLevel.contains(spell));
			newLevel.add(spell);
		}	
		Collections.sort(newLevel);
		spellList.add(newLevel);
	}

	/**
	*  Compute maximum spell level for this book.
	*  Minimum of spells database and wizard level access.
	*/
	int computeMaxLevel () {
		int maxTable = SpellIndex.getInstance().getMaxLevel();
		int maxWizard = LevelAccessTable.getInstance().getMaxSpellLevel(wizLevel);
		return Math.min(maxTable, maxWizard);
	}

	/**
	*  Describe one spell level as a string.
	*/
	public String levelToString (int level) {
		String s = "";
		ArrayList<Spell> list = spellList.get(level - 1);
		for (int i = 0; i < list.size(); i++) {
			s += list.get(i).getName();
			if (i < list.size() - 1)
				s += ", ";			
		}
		return s;	
	}

	/**
	*  Identify this spellbook as a string.
	*/
	public String toString() {
		String s = "";
		for (int level = 1; level <= spellList.size(); level++) {
			s += ordinalDesc(level) + ": ";
			s += levelToString(level);
			if (level < spellList.size())
				s += "; ";
		}
		s += ".";
		return s;
	}

	/**
	*  Get ordinal number description.
	*/
	String ordinalDesc (int num) {
		switch (num) {
			case 1: return "1st";
			case 2: return "2nd";
			case 3: return "3rd";
			default: return num + "th";
		}
	}
	
	/**
	*  Main test function.
	*/
	public static void main (String[] args) {
		Dice.initialize();
		Spellbook book = new Spellbook(6, 9);
		System.out.println(book);
	}
}

